/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: Component.java,v 1.12 2007/05/19 02:45:23 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.component;

import java.util.ArrayList;
import java.util.List;

import edu.indiana.extreme.xbaya.graph.ControlPort;
import edu.indiana.extreme.xbaya.graph.DataPort;
import edu.indiana.extreme.xbaya.graph.EPRPort;
import edu.indiana.extreme.xbaya.graph.Graph;
import edu.indiana.extreme.xbaya.graph.Node;
import edu.indiana.extreme.xbaya.graph.impl.NodeImpl;

/**
 * @author Satoshi Shirasuna
 */
public abstract class Component {

    protected String name;

    protected String description;

    protected ComponentControlPort controlInPort;

    protected List<ComponentControlPort> controlOutPorts;

    protected ComponentEPRPort eprPort;

    /**
     * Creates a Component.
     */
    public Component() {
        this.controlOutPorts = new ArrayList<ComponentControlPort>();
    }

    /**
     * Sets the name of the component.
     * 
     * @param name
     *            The name
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Returns the name of the component.
     * 
     * @return The name of the component
     */
    public String getName() {
        return this.name;
    }

    /**
     * Sets the description of the component.
     * 
     * @param description
     *            The description
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * Returns the description of the component.
     * 
     * @return The description of the component
     */
    public String getDescription() {
        return this.description;
    }

    /**
     * @return The list of input ComponentDataPort
     */
    public abstract List<? extends ComponentDataPort> getInputPorts();

    /**
     * @return The list of output ComponentDataPort
     */
    public abstract List<? extends ComponentDataPort> getOutputPorts();

    /**
     * Returns the index'th input component port.
     * 
     * @param index
     *            The index
     * @return The component port
     */
    public ComponentPort getInputPort(int index) {
        return getInputPorts().get(index);
    }

    /**
     * Returns the index'th output component port.
     * 
     * @param index
     *            The index
     * @return The component port
     */
    public ComponentPort getOutputPort(int index) {
        List<? extends ComponentPort> outs = getOutputPorts();
        if (index >= outs.size()) {
            throw new IllegalArgumentException("index, " + index
                    + ", must be less than " + outs.size() + ".");
        }
        return outs.get(index);
    }

    /**
     * @return The controlInPort.
     */
    public ComponentPort getControlInPort() {
        return this.controlInPort;
    }

    /**
     * @return The list of controlOutPorts.
     */
    public List<ComponentControlPort> getControlOutPorts() {
        return this.controlOutPorts;
    }

    /**
     * @param index
     * @return The controlOutPort at the specified index.
     */
    public ComponentControlPort getControlOutPort(int index) {
        return getControlOutPorts().get(index);
    }

    /**
     * @return The EPR Port
     */
    public ComponentEPRPort getEPRPort() {
        return this.eprPort;
    }

    /**
     * Creates a new node from this component in a specified graph.
     * 
     * @param graph
     *            The specified graph
     * 
     * @return The Node created
     */
    abstract public Node createNode(Graph graph);

    protected void createPorts(NodeImpl node) {
        for (ComponentDataPort input : getInputPorts()) {
            DataPort port = input.createPort();
            node.addInputPort(port);
        }

        for (ComponentDataPort output : getOutputPorts()) {
            DataPort port = output.createPort();
            node.addOutputPort(port);
        }

        if (this.controlInPort != null) {
            ControlPort port = this.controlInPort.createPort();
            node.setControlInPort(port);
        }

        for (ComponentControlPort componentPort : this.controlOutPorts) {
            ControlPort port = componentPort.createPort();
            node.addControlOutPort(port);
        }

        if (this.eprPort != null) {
            EPRPort port = this.eprPort.createPort();
            node.setEPRPort(port);
        }
    }

    /**
     * Returns the HTML to show the information of this Component
     * 
     * @return The string representation of the HTML
     */
    abstract public String toHTML();
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
